package com.alterdekim.game.component;

import com.alterdekim.flash.decompiler.FlashDecompiler;
import com.alterdekim.flash.decompiler.ShockwaveFile;
import com.alterdekim.flash.decompiler.compiler.FlashCompiler;
import com.alterdekim.flash.decompiler.mapper.FlashMapper;
import com.alterdekim.flash.decompiler.mapper.OutputObject;
import com.alterdekim.flash.decompiler.tag.DoAction;
import com.alterdekim.flash.decompiler.tag.TagType;
import com.alterdekim.flash.decompiler.translator.Flash2Java;
import com.alterdekim.flash.decompiler.translator.Java2Flash;
import com.alterdekim.game.component.game.BodyPartType;
import com.alterdekim.game.config.ServerConfig;
import com.alterdekim.game.entity.*;
import com.alterdekim.game.repository.BodyPartRepository;
import com.alterdekim.game.repository.SmileRepository;
import com.alterdekim.game.service.*;
import com.alterdekim.game.storage.StorageProperties;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import org.javatuples.Pair;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.event.ContextRefreshedEvent;
import org.springframework.context.event.EventListener;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.StandardOpenOption;
import java.util.*;

@Slf4j
@Component
public class StartUpListener {

    private static final String ADMIN_USERNAME = "admin";

    @Getter
    private long startTime;

    @Autowired
    private ServerConfig config;

    @Autowired
    private StorageProperties storageProperties;

    @Autowired
    private UserService userService;

    @Autowired
    private MRTService mrtService;

    @Autowired
    private MRService mrService;

    @Autowired
    private TRService trService;

    @Autowired
    private ROService roService;

    @Autowired
    private GoodsService goodsService;

    @Autowired
    private GoodTagService goodTagService;

    @Autowired
    private CatalogService catalogService;

    @Autowired
    private GoodMagicService goodMagicService;

    @Autowired
    private GoodSmileService goodSmileService;

    @Autowired
    private GCDService gcdService;

    @Autowired
    private MAService maService;

    @Autowired
    private AchievementService achievementService;

    @Autowired
    private BackgroundService backgroundService;

    @Autowired
    private LibraryService libraryService;

    @Autowired
    private SportInventoryService sportInventoryService;

    @Autowired
    private VisaService visaService;

    @Autowired
    private MissionService missionService;

    @Autowired
    private GAAccountService gaAccountService;

    @Autowired
    private PriceExchangeService priceExchangeService;

    @Autowired
    private PriceLicenseService priceLicenseService;

    @Autowired
    private PhoneIconService phoneIconService;

    @Autowired
    private GameIconService gameIconService;

    @Autowired
    private IncompatibleService incompatibleService;

    @Autowired
    private LevelExperienceService levelExperienceService;

    @Autowired
    private SmileService smileService;

    @Autowired
    private AvatarInventoryService avatarInventoryService;

    @Autowired
    private LocationService locationService;

    @Autowired
    private PreloaderService preloaderService;

    @Autowired
    private PromotionService promotionService;

    @Autowired
    private PromotionBannerService promotionBannerService;

    @EventListener
    public void onApplicationEvent(ContextRefreshedEvent event) {
        this.startTime = System.currentTimeMillis();

        // todo: compile other swf's
        if( userService.findByUsername(ADMIN_USERNAME) != null ) {

            // todo: remove hardcoded cache folder
            new File("cache/").mkdirs();

            Map<String, Object> root = new HashMap<>();
            Map<String, Object> mr = new HashMap<>();
            mr.put("MR", FlashMapper.objToMap(mrService.getAll()));
            mr.put("RO", FlashMapper.objToMap(roService.getAll()));
            mr.put("TR", FlashMapper.objToMap(trService.getAll()));
            mr.put("MRT", FlashMapper.simpleToMap(mrtService.getAll()));
            root.put("resources", mr);

            try { // todo: and here too ( abolish hardcoded paths )
                Files.write(Path.of("cache/resources.swf"), new FlashCompiler(new Java2Flash().convert(root)).compile(), StandardOpenOption.CREATE);
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        }
        // todo: compile base swf with some changes (make smiles work)


        if( userService.findByUsername(ADMIN_USERNAME) != null ) return;
        String pwd = UUID.randomUUID().toString();
        log.info("Your admin account password is: {}\nChange it later.", pwd);
        userService.saveUser(ADMIN_USERNAME, pwd);

        log.info("Moving pre-compiled data to our database.");
        try {
            processResources(storageProperties.getLocation() + File.separator + config.getDefaultResourcesPath());
            processBase(storageProperties.getLocation() + File.separator + config.getDefaultBasePath());
            // todo: implements miniquests
            String miniquests = storageProperties.getLocation() + File.separator + config.getDefaultMiniquestsPath();
            processCatalogs(storageProperties.getLocation() + File.separator + config.getDefaultCatalogsPath());
        } catch (IOException e) {
            log.error("Unable to move pre-compiled data to the server's database: {}", e.getMessage());
        }
        log.info("Pushing default data blobs to the database");
        pushPreloaders();
        pushPromotions();
        pushPromotionBanners();

        pushBodyParts();
        pushLocations();

        pushPhoneSkins();
        log.info("Initialization complete");
    }

    private void pushPhoneSkins() {
        phoneIconService.findAll()
                        .stream()
                        .map(i -> Pair.with(i.getId(), mrService.findById(i.getIconMRId().longValue())))
                        .filter(i -> i.getValue1().isPresent())
                        .map(i -> Pair.with(i.getValue0(), i.getValue1().get()))
                        .filter(i -> this.mrService.findByUrl(i.getValue1().getUrl().replace("-icon.swf", "-skin.swf"), 26).isPresent())
                        .forEach(i ->
                            this.phoneIconService.save(
                                    new PhoneSkin(
                                            i.getValue0(),
                                            this.mrService.findByUrl(i.getValue1().getUrl().replace("-icon.swf", "-skin.swf"), 26).get().getId()
                                    )
                            )
                        );
    }

    private void pushPromotionBanners() {
        promotionBannerService.savePromotionBanner(new PromotionBanner(9841, true));
        promotionBannerService.savePromotionBanner(new PromotionBanner(9955, true));
        promotionBannerService.savePromotionBanner(new PromotionBanner(12512, true));
    }

    private void pushPromotions() {
        promotionService.savePromotion(new Promotion(5778, "1", true));
        promotionService.savePromotion(new Promotion(5891, "1", true));
        promotionService.savePromotion(new Promotion(9818, "1", true));
    }

    private void pushPreloaders() {
        preloaderService.savePreloader(new Preloader(6857, 2000, true));
        preloaderService.savePreloader(new Preloader(11774, 2000, true));
        preloaderService.savePreloader(new Preloader(11994, 2000, true));
    }

    private void pushLocations() {
        locationService.addLocation(new Location(47l, 3849l, 400.0d, 250.0d, "Площадь у больнички", true, true));
    }

    private void pushBodyParts() {
        List<BodyPart> l = new ArrayList<>();

        l.add(new BodyPart(56L, 56, true, BodyPartType.Body));
        l.add(new BodyPart(60L, 60, true, BodyPartType.Body));
        l.add(new BodyPart(64L, 64, true, BodyPartType.Body));
        l.add(new BodyPart(69L, 69, true, BodyPartType.Body));
        l.add(new BodyPart(73L, 73, true, BodyPartType.Body));
        l.add(new BodyPart(77L, 77, true, BodyPartType.Body));
        l.add(new BodyPart(83L, 83, true, BodyPartType.Body));
        l.add(new BodyPart(89L, 89, true, BodyPartType.Body));

        l.add(new BodyPart(55L, 55, true, BodyPartType.Legs));
        l.add(new BodyPart(62L, 62, true, BodyPartType.Legs));
        l.add(new BodyPart(66L, 66, true, BodyPartType.Legs));
        l.add(new BodyPart(71L, 71, true, BodyPartType.Legs));
        l.add(new BodyPart(75L, 75, true, BodyPartType.Legs));
        l.add(new BodyPart(80L, 80, true, BodyPartType.Legs));
        l.add(new BodyPart(86L, 86, true, BodyPartType.Legs));
        l.add(new BodyPart(92L, 92, true, BodyPartType.Legs));

        l.add(new BodyPart(78L, 78, true, BodyPartType.Ears));
        l.add(new BodyPart(84L, 84, true, BodyPartType.Ears));
        l.add(new BodyPart(90L, 90, true, BodyPartType.Ears));
        l.add(new BodyPart(95L, 95, true, BodyPartType.Ears));
        l.add(new BodyPart(96L, 96, true, BodyPartType.Ears));
        l.add(new BodyPart(97L, 97, true, BodyPartType.Ears));

        l.add(new BodyPart(59L, 59, false, BodyPartType.Eyes));
        l.add(new BodyPart(61L, 61, false, BodyPartType.Eyes));
        l.add(new BodyPart(65L, 65, false, BodyPartType.Eyes));
        l.add(new BodyPart(70L, 70, false, BodyPartType.Eyes));
        l.add(new BodyPart(74L, 74, false, BodyPartType.Eyes));
        l.add(new BodyPart(79L, 79, false, BodyPartType.Eyes));
        l.add(new BodyPart(85L, 85, false, BodyPartType.Eyes));
        l.add(new BodyPart(91L, 91, false, BodyPartType.Eyes));

        l.add(new BodyPart(58L, 58, false, BodyPartType.Nose));
        l.add(new BodyPart(68L, 68, false, BodyPartType.Nose));
        l.add(new BodyPart(82L, 82, false, BodyPartType.Nose));
        l.add(new BodyPart(88L, 88, false, BodyPartType.Nose));
        l.add(new BodyPart(94L, 94, false, BodyPartType.Nose));

        l.add(new BodyPart(57L, 57, false, BodyPartType.Mouth));
        l.add(new BodyPart(67L, 67, false, BodyPartType.Mouth));
        l.add(new BodyPart(81L, 81, false, BodyPartType.Mouth));
        l.add(new BodyPart(87L, 87, false, BodyPartType.Mouth));
        l.add(new BodyPart(93L, 93, false, BodyPartType.Mouth));

        l.add(new BodyPart(63L, 63, false, BodyPartType.Beak));
        l.add(new BodyPart(72L, 72, false, BodyPartType.Beak));
        l.add(new BodyPart(76L, 76, false, BodyPartType.Beak));

        l.add(new BodyPart(99L, 99, false, BodyPartType.Horn));
        l.add(new BodyPart(100L, 100, false, BodyPartType.Horn));
        l.add(new BodyPart(101L, 101, false, BodyPartType.Horn));
        l.add(new BodyPart(102L, 102, false, BodyPartType.Horn));
        l.add(new BodyPart(103L, 103, false, BodyPartType.Horn));

        l.add(new BodyPart(457L, 457, false, BodyPartType.Caps));
        l.add(new BodyPart(458L, 458, false, BodyPartType.Caps));
        l.add(new BodyPart(459L, 459, false, BodyPartType.Caps));
        l.add(new BodyPart(460L, 460, false, BodyPartType.Caps));
        l.add(new BodyPart(461L, 461, false, BodyPartType.Caps));
        l.add(new BodyPart(462L, 462, false, BodyPartType.Caps));
        l.add(new BodyPart(463L, 463, false, BodyPartType.Caps));
        l.add(new BodyPart(464L, 464, false, BodyPartType.Caps));

        l.forEach(i -> avatarInventoryService.saveBodyPart(i));
    }

    private Map<String, Object> decompile(String path) throws IOException {
        FlashDecompiler decompiler = new FlashDecompiler();
        ShockwaveFile file = decompiler.loadFromFile(new File(path));
        DoAction da = (DoAction)file.getTags().stream().filter((p) -> p.getType() == TagType.DoAction).findFirst().get();
        return new Flash2Java(da).convert();
    }

    private void processResources(String swf) throws IOException {
        log.info("Processing resources");

        Map<String, Object> o = decompile(swf);

        FlashMapper.mapToObj(
                o,
                List.of(
                        new OutputObject(MediaResourcePath.class, true, mrtService),
                        new OutputObject(TextResource.class, false, trService),
                        new OutputObject(MediaResource.class, false, mrService),
                        new OutputObject(ResourceObject.class, false, roService)
                )
        );
    }

    private void processCatalogs(String swf) throws IOException {
        log.info("Processing catalogs");
        Map<String, Object> o = decompile(swf);

        FlashMapper.mapToObj(
                o,
                List.of(
                        new OutputObject(GoodTag.class, true, goodTagService),
                        new OutputObject(Good.class, false, goodsService),
                        new OutputObject(Catalog.class, false, catalogService),
                        new OutputObject(GoodMagic.class, false, goodMagicService),
                        new OutputObject(GoodSmile.class, false, goodSmileService),
                        new OutputObject(GoodClubDancefloor.class, false, gcdService)
                )
        );
    }

    // todo: implement other stuff from base.swf
    private void processBase(String swf) throws IOException {
        log.info("Processing base");

        Map<String, Object> o = decompile(swf);

        FlashMapper.mapToObj(
                o,
                List.of(
                    new OutputObject(MagicAbility.class, false, maService),
                    new OutputObject(Achievement.class, false, achievementService),
                    new OutputObject(Background.class, true, backgroundService),
                    new OutputObject(Library.class, false, libraryService),
                    new OutputObject(SportInventory.class, true, sportInventoryService),
                    new OutputObject(Visa.class, false, visaService),
                    new OutputObject(CatalogObject.class, false, catalogService),
                    new OutputObject(Mission.class, false, missionService),
                    new OutputObject(GAAccount.class, true, gaAccountService),
                    new OutputObject(PriceExchange.class, false, priceExchangeService),
                    new OutputObject(PriceLicense.class, false, priceLicenseService),
                    new OutputObject(PhoneIcon.class, true, phoneIconService),
                    new OutputObject(GameIcon.class, true, gameIconService),
                    new OutputObject(Incompatible.class, false, incompatibleService),
                    new OutputObject(LevelExperience.class, true, levelExperienceService),
                    new OutputObject(Smile.class, false, smileService)
                )
        );
    }
}