package com.alterdekim.game.component.rtmp;


import com.alterdekim.game.message.ByteMessage;
import com.alterdekim.game.message.SerializerUtils;
import com.alterdekim.game.message.serializer.BinaryMessageSerializer;
import com.alterdekim.game.utils.ByteUtils;
import lombok.extern.slf4j.Slf4j;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.IntStream;

@Slf4j
public class HandshakeProcessor extends ConnectionProcessor {

    private HandshakeState handshakeState;

    public HandshakeProcessor(InputStream inputStream, OutputStream outputStream, Socket sock) {
        super(inputStream, outputStream, sock);
        this.handshakeState = HandshakeState.C0S0;
    }

    @Override
    public ConnectionState getState() {
        return ConnectionState.HANDSHAKE;
    }

    @Override
    public ConnectionState process() throws IOException {
        switch (handshakeState) {
            case C0S0:
                processFirstMessage();
                break;
            case C1S1:
                processSecondMessage();
                break;
            case C2S2:
                return processThirdMessage();
        }
        return getState();
    }

    private void processFirstMessage() throws IOException {
        byte[] b = new byte[1];
        if( this.getInputStream().read(b) == -1 || b[0] != 0x03 ) {close(); return;}
        this.getOutputStream().write(b);
        this.handshakeState = HandshakeState.C1S1;
    }

    private void processSecondMessage() throws IOException {
        byte[] b = new byte[1536];
        if( this.getInputStream().read(b) == -1 ) { close(); return; }
        this.getOutputStream().write(b);
        this.handshakeState = HandshakeState.C2S2;
    }

    private ConnectionState processThirdMessage() throws IOException {
        byte[] b = new byte[1536];
        if( this.getInputStream().read(b) == -1 ) { close(); return ConnectionState.HANDSHAKE; }
        List<Byte> response = new ArrayList<>();
        IntStream.range(0, 1536).forEach(i -> response.add( (byte) ((Math.random() * 253) + 1) ) );
        write(response);
        return ConnectionState.CONNECTED;
    }

    private enum HandshakeState {
        C0S0,
        C1S1,
        C2S2
    }
}
