package com.alterdekim.game.service;

import com.alterdekim.game.component.ReferenceLoader;
import com.alterdekim.game.component.game.*;
import com.alterdekim.game.component.game.inventory.InventoryGroup;
import com.alterdekim.game.component.game.inventory.InventoryItem;
import com.alterdekim.game.controller.result.signup.AvatarBodyPart;
import com.alterdekim.game.controller.result.signup.AvatarBodyPartGroup;
import com.alterdekim.game.entity.*;
import com.alterdekim.game.repository.*;
import com.alterdekim.game.utils.GameUtils;
import lombok.extern.slf4j.Slf4j;
import org.javatuples.Pair;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.parameters.P;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

@Slf4j
@Service
public class AvatarInventoryService {

    @Autowired
    private ReferenceLoader referenceLoader;

    @Autowired
    private AvatarInventoryRepository inventoryRepository;

    @Autowired
    private GoodRepository goodRepository;

    @Autowired
    private BodyPartRepository bodyPartRepository;

    @Autowired
    private GoodMagicRepository goodMagicRepository;

    @Autowired
    private ResourceService resourceService;

    @Autowired
    private AchievementService achievementService;

    @Autowired
    private SmileService smileService;

    @Autowired
    private BackgroundService backgroundService;

    @Autowired
    private PhoneIconService phoneIconService;

    public void setDefault(Long userId, Long goodId, AvatarInventoryType type) {
        this.inventoryRepository.resetUsed(userId, type);
        this.inventoryRepository.setUsed(userId, goodId, type);
    }

    public InventoryGroup getBodyPartInventoryGroup(Long userId, BodyPartType type) {
        return new InventoryGroup(
                type.getValue(),
                type.getTextResourceId(),
                false,
                type.getMediaResourceId(),
                type.getLayerId(),
                bodyPartRepository.findInventoryItems(userId, type)
        );
    }

    public InventoryGroup getClothInventoryGroup(Long userId, GoodClothType type) {
        return new InventoryGroup(
                type.getVal(),
                type.getTextResourceId(),
                false,
                0,
                type.getLayerId(),
                inventoryRepository.findInventoryItems(userId, type)
        );
    }

    public List<BodyAvatarItem> getWholeAvatarByUserId(Long userId) {
        List<BodyAvatarItem> items = new ArrayList<>();
        items.addAll(
                this.inventoryRepository.findUsedItemsByUserIdAndType(userId, AvatarInventoryType.BodyParts)
                        .stream()
                        .filter(AvatarInventory::getIsUsed)
                        .map(i -> Pair.with(i.getColor(), bodyPartRepository.findById(i.getGoodId())))
                        .filter(i -> i.getValue1().isPresent())
                        .map(i -> Pair.with(i.getValue0(), i.getValue1().get()))
                        .map(i -> new BodyAvatarItem(i.getValue1().getType().getValue(), i.getValue1().getId(), i.getValue0(), true, i.getValue1().getId().intValue(), 0l, i.getValue1().getMediaResourceId(), i.getValue1().getType().getLayerId(), null))
                        .collect(Collectors.toList())
        );
        items.addAll(
                this.inventoryRepository.findUsedItemsByUserIdAndType(userId, AvatarInventoryType.Clothes)
                        .stream()
                        .filter(AvatarInventory::getIsUsed)
                        .map(AvatarInventory::getGoodId)
                        .map(id -> goodRepository.findById(id))
                        .filter(Optional::isPresent)
                        .map(Optional::get)
                        .map(g -> new BodyAvatarItem(null, g.getId(), null, false, 0, g.getId(), g.getMRId(), g.getLayerId(), g.getGoodTypeId()))
                        .collect(Collectors.toList())
        );
        return items;
    }

    public List<BodyAvatarItem> getAvatarItemsByUserId(Long userId) {
        List<BodyAvatarItem> items = new ArrayList<>();
        items.addAll(
                this.inventoryRepository.findUsedItemsByUserIdAndType(userId, AvatarInventoryType.BodyParts)
                    .stream()
                    .map(i -> Pair.with(i.getColor(), bodyPartRepository.findById(i.getGoodId())))
                    .filter(i -> i.getValue1().isPresent())
                    .map(i -> Pair.with(i.getValue0(), i.getValue1().get()))
                    .map(i -> new BodyAvatarItem(i.getValue1().getType().getValue(), i.getValue1().getId(), i.getValue0(), true, i.getValue1().getId().intValue(), 0l, i.getValue1().getMediaResourceId(), i.getValue1().getType().getLayerId(), null))
                    .collect(Collectors.toList())
        );
        items.addAll(
                this.inventoryRepository.findUsedItemsByUserIdAndType(userId, AvatarInventoryType.Clothes)
                        .stream()
                        .map(AvatarInventory::getGoodId)
                        .map(id -> goodRepository.findById(id))
                        .filter(Optional::isPresent)
                        .map(Optional::get)
                        .map(g -> new BodyAvatarItem(null, g.getId(), null, false, 0, g.getId(), g.getMRId(), g.getLayerId(), g.getGoodTypeId()))
                        .collect(Collectors.toList())
        );
        return items;
    }

    public List<InitMagicAbility> getMagicAbilitiesItemsByUserId(Long userId) {
        return this.inventoryRepository.findItemsByUserIdAndType(userId, AvatarInventoryType.Magic)
                .stream()
                .map(AvatarInventory::getGoodId)
                .map(id -> goodMagicRepository.findById(id))
                .filter(Optional::isPresent)
                .map(Optional::get)
                .map(g -> new InitMagicAbility(g.getMagicAbilityId(), "", false)) // todo: implement expiration ability
                .collect(Collectors.toList());
    }

    public List<Smile> getSmilesByUserId(Long userId) {
        return this.inventoryRepository.findItemsByUserIdAndType(userId, AvatarInventoryType.Smile)
                .stream()
                .map(AvatarInventory::getGoodId)
                .map(id -> smileService.findById(id))
                .filter(Optional::isPresent)
                .map(Optional::get)
                .collect(Collectors.toList());
    }

    public List<InventoryItem> getBackgroundsByUserId(Long userId) {
        return this.inventoryRepository.findBackgrounds(userId);
    }

    public List<InventoryItem> getPhonesByUserId(Long userId) {
        return this.inventoryRepository.findPhones(userId);
    }

    public void addPhoneToInventory(Long userId, Long phoneId) {
        this.inventoryRepository.save(new AvatarInventory(referenceLoader.getUserReference(userId), phoneId, true, AvatarInventoryType.Phones, 0));
    }

    public void addBackgroundToInventory(Long userId, Long backgroundId) {
        this.inventoryRepository.save(new AvatarInventory(referenceLoader.getUserReference(userId), backgroundId, true, AvatarInventoryType.Backgrounds, 0));
    }

    public void saveBodyPart(BodyPart bodyPart) {
        this.bodyPartRepository.save(bodyPart);
    }

    private List<AvatarBodyPart> getAvatarBodyPartsByType(BodyPartType type) {
        return this.bodyPartRepository.findByBodyPartType(type)
                .stream()
                .filter(b -> resourceService.getUrlByMediaResourceId(b.getMediaResourceId().longValue()).isPresent())
                .map(b -> new AvatarBodyPart(
                                b.getId().intValue(),
                                0,
                                false,
                                resourceService.getUrlByMediaResourceId(b.getMediaResourceId().longValue()).get()
                        )
                )
                .collect(Collectors.toList());
    }

    public Long getSelectedPhoneId(Long userId) {
        return this.inventoryRepository.findUsedItemsByUserIdAndType(userId, AvatarInventoryType.Phones)
                .stream()
                .map(AvatarInventory::getGoodId)
                .findFirst().orElse(0L);
    }

    public Long getUsedBackground(Long userId) {
        return this.inventoryRepository.findUsedItemsByUserIdAndType(userId, AvatarInventoryType.Backgrounds)
                .stream()
                .map(AvatarInventory::getGoodId)
                .findFirst().orElse(0L);
    }

    public String getAchievementsByUserId(Long userId) {
        return this.inventoryRepository.findItemsByUserIdAndType(userId, AvatarInventoryType.Achievments)
                .stream()
                .map(a -> achievementService.findById(a.getGoodId()))
                .filter(Optional::isPresent)
                .map(Optional::get)
                .map(a -> a.getId().toString())
                .collect(Collectors.joining(","));
    }

    private List<AvatarBodyPartGroup> getAvatarBodyPartGroups(boolean isShape) {
        List<AvatarBodyPartGroup> groups = new ArrayList<>();
        for( BodyPartType type : BodyPartType.values() ) {
            if( type.isShape() != isShape ) continue;
            groups.add(
                    new AvatarBodyPartGroup(
                            type.getValue(),
                            type.name(),
                            resourceService.getUrlByMediaResourceId(type.getMediaResourceId()).get(),
                            type.getLayerId(),
                            isShape ? this.getAvatarBodyPartsByType(type) : Stream.of(
                                    this.getAvatarBodyPartsByType(type),
                                    Collections.singletonList(
                                            new AvatarBodyPart(98, 0, true, this.resourceService.getUrlByMediaResourceId(98L).get())
                                    )
                            )
                            .flatMap(Collection::stream)
                            .collect(Collectors.toList())
                    )
            );
        }
        return groups;
    }

    public List<AvatarBodyPartGroup> getShapeGroups() {
        return this.getAvatarBodyPartGroups(true);
    }

    public List<AvatarBodyPartGroup> getBodyPartGroups() {
        return this.getAvatarBodyPartGroups(false);
    }

    private List<AvatarBodyPart> getRandomStuffByGoodType(GoodClothType type) {
        List<Good> g = this.goodRepository.findByGoodTypeId(type.getVal());
        int l = (int) Math.round(Math.random() * (g.size() - 4));
        return g.subList(l, l+3)
                .stream()
                .map(i -> new AvatarBodyPart(
                        i.getId().intValue(),
                        0,
                        false,
                        resourceService.getUrlByMediaResourceId(i.getMRId().longValue()).get()
                    )
                )
                .collect(Collectors.toList());
    }

    public List<AvatarBodyPartGroup> getAvatarInventoryGroups() {
        List<AvatarBodyPartGroup> groups = new ArrayList<>();
        for( GoodClothType type : GoodClothType.values() ) {
            groups.add(
                    new AvatarBodyPartGroup(
                            type.getVal(),
                            type.name(),
                            resourceService.getUrlByMediaResourceId(type.getTabMediaResourceId()).get(),
                            type.getLayerId(),
                            Stream.of(
                                    this.getRandomStuffByGoodType(type),
                                    Collections.singletonList(
                                            new AvatarBodyPart(98, 0, true, this.resourceService.getUrlByMediaResourceId(98L).get())
                                    )
                            )
                            .flatMap(Collection::stream)
                            .collect(Collectors.toList())
                    )
            );
        }
        return groups;
    }

    public void addGoodToInventory(AvatarInventory avatarInventory) {
        this.inventoryRepository.save(avatarInventory);
    }
}
